﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using Microsoft.AspNetCore.Mvc;
using DWNet.Data;
using OrderDemo.Services;
using Microsoft.AspNetCore.Http;
using SnapObjects.Data;

namespace OrderDemo.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class WOrderMainController : ControllerBase
    {
        private readonly IWOrderMainService _service;

        public WOrderMainController(IWOrderMainService service)
        {
            _service = service;
        }

        // GET api/WOrderMain/DwOrderItemUeRetrieve
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<string> DwOrderItemUeRetrieve(string custNo, string orderNo)
        {
            try
            {
                IDataStore orderItems = _service.DwOrderItemUeRetrieve(custNo, orderNo);

                // Export data from the Primary buffer and DDDWs.
                // Return Status Code 200 
                return Ok(orderItems.ExportJson(true, false, false, true, MappingMethod.Index));
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }

        // GET api/WOrderMain/DwOrderUeRetrieve
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        public ActionResult<IDataStore> DwOrderUeRetrieve(string custNo)
        {
            try
            {
                IDataStore orders = _service.DwOrderUeRetrieve(custNo);

                // Return Status Code 200 with the Plain JSON
                return Ok(orders);
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }

        // DELETE api/WOrderMain/UeDeleteOrder/{orderNo}
        [HttpDelete("{orderNo}")]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult UeDeleteOrder(string orderNo)
        {
            // Do a Try/Catch for Exception handling
            try
            {
                // Delete the Order and Order Items by calling the service
                _service.UeDeleteOrder(orderNo);

                // Return a Status Code of 200
                return Ok();
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }

        // GET api/WOrderMain/UePostOpen
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<IDataStore> UePostOpen()
        {
            // Do a Try/Catch for Exception handling
            try
            {
                // Retrieve data
                IDataStore customers = _service.UePostOpen();

                // Return a Status Code of 200
                return Ok(customers);
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }
    }
}
