﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using Microsoft.AspNetCore.Mvc;
using DWNet.Data;
using OrderDemo.Services;
using Microsoft.AspNetCore.Http;
using SnapObjects.Data;
using OrderDemo.Models;

namespace OrderDemo.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class WOrderModifyController : ControllerBase
    {
        // Declare the local variable for the Service.
        private readonly IWOrderModifyService _service;

        // Constructor.
        public WOrderModifyController(IWOrderModifyService service)
        {
            // Initialize the Service variable
            _service = service;
        }

        // POST api/WOrderNew/DwOrderItemItemChanged
        [HttpPost]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<IDataPacker> DwOrderItemItemChanged(IDataUnpacker dataUnpacker)
        {
            string column = dataUnpacker.GetValue<string>("Column");
            string data = dataUnpacker.GetValue<string>("Data");

            // Do a Try/Catch for Exception handling
            try
            {
                // Execute the business logic
                short result = _service.DwOrderItemItemChanged(
                    column, data, out bool hasProduct, out D_Product product);

                // Add values to an IDataPacker object
                IDataPacker packer = new DataPacker();
                packer.AddValue("Result", result);
                packer.AddValue("HasProduct", hasProduct);

                // Success
                if (result == 1)
                {
                    packer.AddValue("fcategory", product.Fcategory);
                    packer.AddValue("fproname", product.Fproname ?? "");
                    packer.AddValue("fsku", product.Fsku ?? "");
                    packer.AddValue("funit_price", product.Funit_Price);
                    packer.AddValue("fdescription", product.Fdescription);
                }

                // Return a Status Code of 200
                return Ok(packer);
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }

        // GET api/WOrderModify/Open
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<IDataPacker> Open(string custNo, string orderNo)
        {
            try
            {
                _service.Open(custNo, orderNo, out IDataStore order, out IDataStore orderItems);

                IDataPacker packer = new DataPacker();

                packer.AddDataStore("order", order);

                // DataWindow Standard Format (Including DDDWs)
                packer.AddDataStore("orderitems", orderItems, true);

                // Return Status Code 200
                return Ok(packer);
            }
            catch
            {
                // Return a Status Code 500 for Internal Server Errors
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }

        // POST api/WOrderModify/UeSave
        [HttpPost]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult UeSave(IDataUnpacker dataUnpacker)
        {
            // Declare and initialize local .NET DataStore variables
            // By getting the Packaged DataStores
            IDataStore order = dataUnpacker.GetDataStore("order");
            IDataStore orderItems = dataUnpacker.GetDataStore("orderitems");

            // Check if the modification was successful
            if (_service.UeSave(order, orderItems))
            {
                // Return Status Code 200
                return Ok();
            }
            else
            {
                // Return a Status Code 500 for Internal Server Error
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }

        // GET api/WOrderModify/WfGetMaxLineIds
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<short> WfGetMaxLineId(string orderNo)
        {
            try
            {
                short maxId = _service.WfGetMaxLineId(orderNo);

                // Return Status Code 200
                return Ok(maxId);
            }
            catch
            {
                // Return a Status Code 500 for Internal Server Errors
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }
    }
}
