﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using Microsoft.AspNetCore.Mvc;
using DWNet.Data;
using OrderDemo.Services;
using Microsoft.AspNetCore.Http;
using SnapObjects.Data;
using OrderDemo.Models;

namespace OrderDemo.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class WOrderNewController : ControllerBase
    {
        // Declare the local variable for the Service.
        private readonly IWOrderNewService _service;

        // Constructor.
        public WOrderNewController(IWOrderNewService service)
        {
            // Initialize the Service variable
            _service = service;
        }

        // POST api/WOrderNew/DwOrderItemChanged
        [HttpPost]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<IDataPacker> DwOrderItemChanged(IDataUnpacker dataUnpacker)
        {
            string column = dataUnpacker.GetValue<string>("Column");
            string data = dataUnpacker.GetValue<string>("Data");

            // Do a Try/Catch for Exception handling
            try
            {
                // Open Event
                short result = _service.DwOrderItemChanged(column, data, out bool hasCustomer, out string orderNo);

                // Add values to a DataPacker
                IDataPacker packer = new DataPacker();
                
                packer.AddValue("Result", result);
                packer.AddValue<bool>("HasCustomer", hasCustomer);

                if (result == 1) 
                { 
                   packer.AddValue<string>("OrderNo", orderNo); 
                }
                

                // Return a Status Code of 200
                return Ok(packer);
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }

        // POST api/WOrderNew/DwOrderItemItemChanged
        [HttpPost]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<IDataPacker> DwOrderItemItemChanged(IDataUnpacker dataUnpacker)
        {
            string column = dataUnpacker.GetValue<string>("Column");
            string data = dataUnpacker.GetValue<string>("Data");

            // Do a Try/Catch for Exception handling
            try
            {
                // Execute the business logic
                short result = _service.DwOrderItemItemChanged(
                    column, data, out bool hasProduct, out D_Product product);

                // Add values to a DataPacker
                IDataPacker packer = new DataPacker();

                packer.AddValue("Result", result);
                packer.AddValue("HasProduct", hasProduct);

                // Success
                if (result == 1)
                {
                    packer.AddValue("fcategory", product.Fcategory);
                    packer.AddValue("fproname", product.Fproname ?? "");
                    packer.AddValue("fsku", product.Fsku ?? "");
                    packer.AddValue("funit_price", product.Funit_Price);
                    packer.AddValue("fdescription", product.Fdescription);
                }

                // Return a Status Code of 200
                return Ok(packer);
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }

        // GET api/WOrderNew/Open
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<IDataPacker> Open()
        {
            // Do a Try/Catch for Exception handling
            try
            {
                // Open Event
                _service.Open(out IDataStore products, out IDataStore categories);

                // Add values to a DataPacker
                IDataPacker packer = new DataPacker();
                packer.AddDataStore("fproname", products);
                packer.AddDataStore("fcategory", categories);

                // Return a Status Code of 200
                return Ok(packer);
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }

        // POST api/WOrderNew/UeSave
        [HttpPost]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult UeSave(IDataUnpacker dataUnpacker)
        {
            // Declare and initialize local .NET DataStore variables
            // By getting the Packaged DataStores
            IDataStore order = dataUnpacker.GetDataStore("order");
            IDataStore orderItems = dataUnpacker.GetDataStore("orderitems");

            // Check if the modification was successful
            if (_service.UeSave(order, orderItems))
            {
                // Return Status Code 200
                return Ok();
            }
            else
            {
                // Return a Status Code 500 for Internal Server Error
                return StatusCode(StatusCodes.Status500InternalServerError);
            }
        }

        /// GET api/WOrderNew/WfGetOrderNo
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status500InternalServerError)]
        public ActionResult<string> WfGetOrderNo(string custNo)
        {
            // Do a Try/Catch for Exception handling
            try
            {
                // Return the Formatted Order Number by calling the service
                return _service.WfGetOrderNo(custNo);
            }
            catch (Exception e)
            {
                // Return a Status Code of 500 with the Exception Message
                return StatusCode(StatusCodes.Status500InternalServerError, e.Message);
            }
        }
    }
}
