﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using DWNet.Data;
using OrderDemo.Models;
using SnapObjects.Data;

namespace OrderDemo.Services.Impl
{
    public class WOrderModifyService : IWOrderModifyService
    {
        // The DataContext local variable. (Similar to PB's SQLCA)
        private readonly AppeonDataContext _context;


        // Initializes a new instance of the WOrderModifyService class.
        public WOrderModifyService(AppeonDataContext Context)
        {
            // Initialize the DataContext variable
            _context = Context;
        }

        public short DwOrderItemItemChanged(string columnName, string data, out bool hasProduct, out D_Product product)
        {
            hasProduct = true;
            product = new D_Product();

            switch (columnName.ToLower())
            {
                case "fsku":
                    // Filled with category and name
                    product = _context.SqlExecutor.SelectOne<D_Product>(
                        @"SELECT Top 1 fcategory,fproname,funit_price,fdescription 
                        FROM t_products 
                        WHERE fsku = @Data", data);

                    if (product == null)
                    {
                        hasProduct = false;
                        return -1;
                    }

                    break;
                case "fproname":
                    // Filled with category and name
                    product = _context.SqlExecutor.SelectOne<D_Product>(
                        @"SELECT Top 1 fcategory, funit_price, fsku, fdescription 
                        FROM t_products 
                        WHERE fproname = @Data", data);

                    if (product == null)
                    {
                        hasProduct = false;
                        return -1;
                    }

                    break;
                default:
                    return 0;
            }

            return 1;
        }

        public void Open(string custNo, string orderNo, out IDataStore order, out IDataStore orderItems)
        {
            order = new DataStore("d_order_modify", _context);
            order.Retrieve(orderNo);

            orderItems = new DataStore("d_orderitem_edit", _context);
            orderItems.Retrieve(custNo, orderNo);
        }

        public bool UeSave(IDataStore order, IDataStore orderItems)
        {
            order.DataContext = _context;
            orderItems.DataContext = _context;

            // Begin the Transaction
            _context.BeginTransaction();

            // Do a try/catch to catch errors
            try
            {
                // Update the Order DataStore
                order.Update();

                // Update the OrderItems DataStore
                orderItems.Update();

                // Commit the transaction
                _context.Commit();

                // Return TRUE for success
                return true;
            }
            catch (Exception)
            {
                // Rollback the transaction
                _context.Rollback();

                // Return FALSE for failure
                return false;
            }
        }

        public short WfGetMaxLineId(string orderNo)
        {
            // Paste the Embedded SQL query in this function
            short maxLineId = _context.SqlExecutor.Scalar<short>(
                @"SELECT isnull(max(flineid),0)  
                FROM t_orders_items 
                WHERE forderNo = @orderNo",
                orderNo);

            // Return the Max Line Id
            return maxLineId;
        }
    }
}
