﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using DWNet.Data;
using OrderDemo.Models;
using SnapObjects.Data;

namespace OrderDemo.Services.Impl
{
    public class WOrderNewService : IWOrderNewService
    {
        // The DataContext local variable. (Similar to PB's SQLCA)
        private readonly AppeonDataContext _context;

        // Initializes a new instance of the WOrderNewService class.
        public WOrderNewService(AppeonDataContext Context)
        {
            // Initialize the DataContext variable
            _context = Context;
        }

        public short DwOrderItemChanged(string columnName, string data, out bool hasCustomer, out string orderNo)
        {
            hasCustomer = true;
            orderNo = "";
            
            switch (columnName.ToLower())
            {
                case "fcustno":
                    int count = _context.SqlExecutor.Scalar<int>(
                        @"SELECT Count(*) FROM t_customers 
                        WHERE fcustno = @Data", data);
                    if (count < 1)
                    {
                        hasCustomer = false;
                        return -1;
                    }

                    // Get new order no
                    orderNo = this.WfGetOrderNo(data);
                    
                    break;
                default:
                    return 0;
            }

            return 1;
        }

        public short DwOrderItemItemChanged(string columnName, string data, out bool hasProduct, out D_Product product)
        {
            hasProduct = true;
            product = new D_Product();

            switch (columnName.ToLower())
            {
                case "fsku":
                    // Filled with category and name
                    product = _context.SqlExecutor.SelectOne<D_Product>(
                        @"SELECT Top 1 fcategory,fproname,funit_price,fdescription 
                        FROM t_products 
                        WHERE fsku = @Data", data);

                    if (product == null)
                    {
                        hasProduct = false;
                        return -1;
                    }

                    break;
                case "fproname":
                    // Filled with category and name
                    product = _context.SqlExecutor.SelectOne<D_Product>(
                        @"SELECT Top 1 fcategory, funit_price, fsku, fdescription 
                        FROM t_products 
                        WHERE fproname = @Data", data);

                    if (product == null)
                    {
                        hasProduct = false;
                        return -1;
                    }

                    break;
                default:
                    return 0;
            }

            return 1;
        }

        public void Open(out IDataStore products, out IDataStore categories)
        {
            // Create the .NET DataStore; set its Dataobject; and set its DataContext
            IDataStore orderItems = new DataStore("d_orderitem_edit", _context);

            // Retrieve data for DataWindowChild fproname
            products = orderItems.GetChild("fproname");
            products.DataContext = _context;
            products.Retrieve();

            // Retrieve data for DataWindowChild fcategory
            categories = orderItems.GetChild("fcategory");
            categories.DataContext = _context;
            categories.Retrieve();
        }

        public bool UeSave(IDataStore order, IDataStore orderItems)
        {
            order.DataContext = _context;
            orderItems.DataContext = _context;

            // Begin the Transaction
            _context.BeginTransaction();

            // Do a try/catch to catch errors
            try
            {
                // Update the Order DataStore
                order.Update();

                // Update the Products DataStore
                orderItems.Update();

                // Commit the transaction
                _context.Commit();

                // Return TRUE for success
                return true;
            }
            catch
            {
                // Rollback the transaction
                _context.Rollback();

                // Return FALSE for failure
                return false;
            }
        }

        public string WfGetOrderNo(string custNo)
        {
            // Paste the Embedded SQL in this function
            int? maxOrderNo = _context.SqlExecutor.Scalar<int?>(
                @"SELECT max(convert(integer,right(forderNo,2)))+1 
                FROM t_orders WHERE fcustNo = @as_custNo",
                custNo);

            // Check if the Order Number exists
            if (maxOrderNo == null || maxOrderNo <= 0)
            {
                // Set the local variable to 1 for NOT VALID
                maxOrderNo = 1;
            }

            // Set the formatted Value of the Order Number
            string orderNo = maxOrderNo.Value.ToString("00");
            orderNo = custNo + orderNo;

            // Return the Formatted Order Number
            return orderNo;
        }
    }
}
